/**
 * Eine moegliche Loesung fuer einen StringTokenizer, bei der direkt im
 * Konstruktor der Token-Strom erzeugt wird.
 * 
 * @author Dietrich Boles
 */
class StringTokenizer { // eigentlich public

	private String str;
	private String delim;
	private String[] token;
	private int index;

	public StringTokenizer(String str, String delim) {
		this.str = str;
		this.delim = delim;
		this.index = 0;
		this.generateTokenStream();
	}

	public StringTokenizer(StringTokenizer tokenizer) {
		this.str = tokenizer.str;
		this.delim = tokenizer.delim;
		this.index = tokenizer.index;
		this.token = new String[this.token.length];
		for (int i = 0; i < this.token.length; i++) {
			this.token[i] = new String(tokenizer.token[i]);
		}
	}

	public boolean equals(StringTokenizer obj) {
		return this.str.equals(obj.str) && this.delim.equals(obj.delim)
				&& this.index == obj.index;
		// der Token-Strom braucht nicht mehr verglichen zu
		// werden; der ist ja direkt abhaengig vom String
		// und Delimiter-String
	}

	public int countToken() {
		return this.token.length;
	}

	public String[] getAllToken() {
		return this.token;
	}

	public String nextToken() {
		if (this.index >= this.token.length) {
			return null;
		}
		return this.token[this.index++];
	}

	public boolean hasMoreToken() {
		return this.index < this.token.length;
	}

	private void generateTokenStream() {
		this.token = new String[0];

		// Token zaehlen
		int number = 0;
		int i = 0;
		while (i < this.str.length()) {
			i = this.findNextTokenIndex(i);
			if (i == this.str.length()) {
				break;
			}
			number++;
			i++;
			i = this.findNextDelimIndex(i);
			if (i == this.str.length()) {
				break;
			}
			i++;
		}

		// Token-Strom aufbauen
		this.token = new String[number];
		int from = 0;
		int to = -1;
		for (i = 0; i < number; i++) {
			from = this.findNextTokenIndex(to + 1);
			to = this.findNextDelimIndex(from + 1);
			this.token[i] = this.str.substring(from, to);
		}
	}

	private int findNextTokenIndex(int abIndex) {
		int i = abIndex;
		while (i < this.str.length()
				&& this.delim.indexOf(this.str.charAt(i)) >= 0) {
			i++;
		}
		if (i >= this.str.length()) {
			return this.str.length();
		} else {
			return i;
		}
	}

	private int findNextDelimIndex(int abIndex) {
		int i = abIndex;
		while (i < this.str.length()
				&& this.delim.indexOf(this.str.charAt(i)) < 0) {
			i++;
		}
		if (i >= this.str.length()) {
			return this.str.length();
		} else {
			return i;
		}
	}
}

public class UE15Aufgabe10 {

	// Testprogramm
	public static void main(String[] args) {

		if (args.length < 2) {
			System.out.println("Fehlende Parameter!");
			System.out.println("1. Parameter: String; 2. Parameter: Delimiter");
			return;
		}

		StringTokenizer tok = new StringTokenizer(args[0], args[1]);
		System.out.println("Anzahl: " + tok.countToken());
		while (tok.hasMoreToken()) {
			System.out.println("Token: " + tok.nextToken());
		}
	}

}
